/*
  Analog Clock Face with NTP with 1.8" TFT (ST7735) running on ESP8266
  NTP code extracted from TimeNTP_ESP8266Wifi.ino included in `Time` by Paul Stoffregen
  (c) 2015 Karl Pitrich, MIT Licensed

  Dependencies:
    * https://github.com/adafruit/Adafruit-GFX-Library.git
    * https://github.com/PaulStoffregen/Time.git
    * Depending on display:
      * https://github.com/nzmichaelh/Adafruit-ST7735-Library.git
      * https://github.com/0xPIT/Adafruit_ILI9340
*/

#include <TimeLib.h> 
#include <ESP8266WiFi.h>
#include <WiFiUdp.h>
#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_ST7735.h>
#include <Adafruit_MCP4725.h>

const float degToRad = 0.0174532925;  // 1° == 0.0174532925rad (PI/180) 

const char ssid[] = "........";            // network SSID
const char pass[] = "..................";  // network password

const int timeZone = 2;               // Central European Time
const char* timerServerDNSName = "0.de.pool.ntp.org";
IPAddress timeServer;

WiFiUDP Udp;
const unsigned int localPort = 8888;  // local port to listen for UDP packets
const int NTP_PACKET_SIZE = 48;       // NTP time is in the first 48 bytes of message
byte packetBuffer[NTP_PACKET_SIZE];   // buffer to hold incoming & outgoing packets
char Buffer[20];
typedef struct Point_s {
  int x;
  int y;
} Point_t;
uint16_t ii=0;
int nul=48;
int first;
int secon;
int Year;
int prevDay=0;
#define TFT_CS     15
#define TFT_RST    12
#define TFT_DC     2

  Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS,  TFT_DC, TFT_RST);
  Point_t displayCenter = { // 160x128 with rotation=3
    ST7735_TFTHEIGHT_18 / 2,
    ST7735_TFTWIDTH / 2
  };

// radius of the clock face
uint16_t clockRadius = std::min(63, 63) - 10;


void drawHourTick(uint8_t hour, Point_t center, uint16_t radius)
{
  float radians = hour * PI / 6.0;

  tft.drawLine(
    63 + (int)((0.91 * radius * sin(radians))),
    63 - (int)((0.91 * radius * cos(radians))),
    63 + (int)((1.00 * radius * sin(radians))),
    63 - (int)((1.00 * radius * cos(radians))),
    ST7735_WHITE
  );
}

void drawMinuteDot(uint8_t minute, Point_t center, uint16_t radius)
{
    float radians = minute * PI / 30.0;

    if (minute % 15 == 0) {
      tft.fillCircle(
        63 + (int)((0.91 * (radius+5) * sin(radians))),
        63 - (int)((0.91 * (radius+5) * cos(radians))),
        2,
        ST7735_WHITE
      );
    }
    else {
      tft.drawPixel(
        63 + (int)((0.91 * radius * sin(radians))),
        63 - (int)((0.91 * radius * cos(radians))),
        ST7735_WHITE
      );
    }
}

void drawClockFace(uint16_t radius, Point_t center) {
  tft.fillScreen(ST7735_BLACK);
  tft.setRotation(0);

  for (uint8_t hr = 1; hr <= 12; hr++) {
    drawHourTick(hr, center, radius);
  }

  for (uint8_t min = 1; min <= 60; min++) {
    drawMinuteDot(min, center, radius);
  }
}

uint8_t lastH, lastM, lastS;

void drawClockHands(time_t now, uint16_t radius, Point_t center) {
  float radians;
  uint8_t hh = hour(now);
  uint8_t mm = minute(now);
  uint8_t ss = second(now);
  auto centerLine = [&](Point_t p, uint16_t c = ST7735_WHITE) {
    tft.drawLine(63, 63, p.x, p.y, c);
  };
  
  auto led = [&](Point_t p, uint16_t c = ST7735_RED) {
    tft.fillCircle(p.x, p.y,3,c);
  };
  
  auto makePoint = [&](float factor) {
    Point_t pt = {
      x: 63 + (uint16_t)(factor * radius * sin(radians)),
      y: 63 - (uint16_t)(factor * radius * cos(radians))
    };
    return std::move(pt);
  };

  // hour
  radians = (lastH % 12) * PI / 6.0 + (PI * lastM / 360.0);
  Point_t pH = makePoint(0.5);
  centerLine(pH, ST7735_BLACK);
  radians = (hh % 12) * PI / 6.0 + (PI * mm / 360.0);
  pH = makePoint(0.5);
  centerLine(pH);

  // minute
  radians = (lastM * PI / 30.0) + (PI * lastS / 1800.0);
  Point_t pM = makePoint(0.7);
  centerLine(pM, ST7735_BLACK);
  radians = (mm * PI / 30.0) + (PI * ss / 1800.0);
  pM = makePoint(0.7);
  centerLine(pM);

  // second
  radians = lastS * PI / 30.0; 
  Point_t pS = makePoint(0.8);
  centerLine(pS, ST7735_BLACK);
  radians = ss * PI / 30.0; 
  pS = makePoint(0.8);
  centerLine(pS, ST7735_RED);

  // second led
  radians = ss * PI / 30.0; 
  Point_t pLED = makePoint(1.13);
  led(pLED, ST7735_RED);
  if (lastS==59)
  for (uint8_t LED = 1; LED <= 59; LED++) {
  radians = LED * PI / 30.0;
  Point_t pLED = makePoint(1.13);
  led(pLED, ST7735_BLACK);
  }
  lastH = hh;
  lastM = mm;
  lastS = ss;

  // center dot
  tft.fillCircle(63, 63, 3, ST7735_RED);
}
Adafruit_MCP4725 dac;
void setup() 
{
  tft.initR(INITR_BLACKTAB);
  Wire.begin(5,4);
  Wire.setClock(400000L);
  dac.begin(0x62);
  drawClockFace(clockRadius, displayCenter);

  WiFi.begin(ssid, pass);
  
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
  }
  
  WiFi.hostByName(timerServerDNSName, timeServer);
  
  Udp.begin(localPort);
  
  setSyncProvider(getNtpTime);
}

time_t prevTime = 0;

void loop()
{
  
    if (timeStatus() != timeNotSet){ 
      time_t current = now();
             
    if (current != prevTime) {
      drawClockHands(current, clockRadius, displayCenter);
      printTime();
      if (prevDay != weekday());{
      printdate();  
      }
      printdate();
      printbin();
      clicker();
      prevTime = current;
      prevDay=weekday();
}
}
}
